"use strict";
const AwsDynamoDB = require("aws-sdk/clients/dynamodb");
const rxjs_1 = require("rxjs");
class DynamoDB {
    constructor(config = {}) {
        this.config = config;
        this.config = Object.assign({}, DynamoDB.defaultConfig, config);
        this.client = new AwsDynamoDB.DocumentClient(this.config);
    }
    create() {
        return (msg, scheduler) => {
            return rxjs_1.Observable.of(msg).flatMap((msg) => {
                if (!msg) {
                    return rxjs_1.Observable.empty();
                }
                let awsOperationStream = rxjs_1.Observable.of(msg).flatMap(() => rxjs_1.Observable.create((observer) => {
                    if (!msg.operation) {
                        return observer.error(new Error('The format of the message for this output stream is invalid'));
                    }
                    let method = this.client[msg.operation];
                    method.call(this.client, msg.params || {}, (error, data) => {
                        if (error) {
                            const isIgnored = (this.config.IGNORE_ERRORS || []).find((err) => error.code === err.code &&
                                (!error.message || error.message === err.message));
                            if (!isIgnored) {
                                return observer.error(new Error(error));
                            }
                        }
                        observer.next(msg);
                        observer.complete();
                    });
                    return () => {
                        method = null;
                    };
                }));
                if (msg.retry) {
                    awsOperationStream = awsOperationStream.retryWhen(error => {
                        return error
                            .do(err => console.error(err))
                            .scan((acc, err) => acc + 1, 0)
                            .flatMap(errorCount => {
                            const retryDelay = msg.retryDelay || (() => rxjs_1.Observable.of(0));
                            let delayStream = retryDelay.call(this, msg);
                            if (!(delayStream && delayStream[Symbol.observable])) {
                                console.warn('retryDelay should return a valid Observable');
                                delayStream = rxjs_1.Observable.of(0);
                            }
                            return delayStream.first().map(delay => [errorCount, delay]);
                        })
                            .flatMap(([count, delay]) => {
                            if (count > msg.retry - 1) {
                                return rxjs_1.Observable.throw(new Error(`Operation failed after ${msg.retry} attempts`));
                            }
                            return rxjs_1.Observable.of(count).delay(delay, scheduler);
                        });
                    });
                }
                return awsOperationStream;
            });
        };
    }
}
DynamoDB.defaultConfig = {
    IGNORE_ERRORS: []
};
(function (DynamoDB) {
    let Operations;
    (function (Operations) {
        Operations["PUT"] = "put";
        Operations["DELETE"] = "delete";
        Operations["UPDATE"] = "update";
    })(Operations = DynamoDB.Operations || (DynamoDB.Operations = {}));
})(DynamoDB || (DynamoDB = {}));
module.exports = DynamoDB;
//# sourceMappingURL=index.js.map